@@ -1501,6 +1501,7 @@ read_csv_header <- function(f, comment.char = '#') {
   niter <- 0
   iter.count <- NA
   save.warmup <- FALSE
+  sample.count <- NA_integer_
   while (length(input <- readLines(con, n = 1)) > 0) {
     niter <- niter + 1
     if (!grepl(comment.char, input)) break;
@@ -1524,7 +1525,7 @@ read_csv_header <- function(f, comment.char = '#') {
     if(save.warmup)
       iter.count <- warmup.count + sample.count
     else
-      iter.count <- warmup.count
+      iter.count <- sample.count
   } 
   attr(header, "iter.count") <- iter.count
   attr(header, "lineno") <- niter
@@ -1685,7 +1686,7 @@ throw_sampler_warnings <- function(object) {
 get_CXX <- function(CXX14 = TRUE) {
   if (.Platform$OS.type != "windows")
     return (system2(file.path(R.home(component = "bin"), "R"),
-            args = paste("CMD config", ifelse(CXX14, "CXX14", "CXX")),
+            args = paste("CMD config", ifelse(CXX14, "CXX14", "CXX11")),
             stdout = TRUE, stderr = FALSE))
 
     ls_path <- Sys.which("ls")
@@ -1706,3 +1707,18 @@ avoid_crash <- function(mod) {
   as(get("packageName", envir = mod)["info"][1], "character") %in% 
     c("<pointer: (nil)>", "<pointer: 0x0>")
 }
+
+make_makevars <- function(DIR = tempdir()) {
+  CXX11 <- get_CXX(FALSE)
+  if (!file.exists(CXX11)) CXX11 <- basename(CXX11)
+  Makevars <- c(CXX14 = paste0("CXX14 := ", CXX11),
+                CXX14STD = "CXX14STD := -std=c++1y",
+                CXX14FLAGS = "CXX14FLAGS := -O3")
+  fn <- file.path(DIR, ifelse(.Platform$OS.type == "windows", 
+                              "Makevars.win", "Makevars"))
+  if (!file.exists(fn) && file.access(DIR, mode = 2) == 0) {
+    writeLines(Makevars, con = fn)
+    return(invisible(TRUE))
+  }
+  return(invisible(FALSE))
+}
